package database

import (
	"database/sql"
)

type Tag struct {
	Type
	id   int64
	name string
}

func InsertTagToDatabase(db *Database, name string) *Tag {

	rows, err := db.Query("INSERT INTO tags (name) VALUES ($1) RETURNING id;", name)
	if err != nil {
		return nil
	}
	defer rows.Close()
	rows.Next()
	var id int64
	err = rows.Scan(&id)
	if err != nil {
		return nil
	}
	tag := GetTagFromDatabase(db, id)

	return tag
}

func GetOrCreateTagFromDatabaseByName(db *Database, name string) (tag *Tag) {
	tag = GetTagFromDatabaseByName(db, name)
	if tag == nil {
		tag = InsertTagToDatabase(db, name)
	}

	return
}

func GetTagFromDatabaseByName(db *Database, name string) *Tag {
	rows, err := db.Query("SELECT id, name FROM tags WHERE name ILIKE $1;", name)
	if err != nil {
		return nil
	}
	defer rows.Close()
	rows.Next()
	r, err := GetTagFromRow(db, rows)
	if err != nil {
		return nil
	}
	return r
}

func GetTagFromDatabase(db *Database, id int64) *Tag {
	rows, err := db.Query("SELECT id, name FROM tags WHERE id = $1;", id)
	if err != nil {
		return nil
	}
	defer rows.Close()
	rows.Next()
	r, err := GetTagFromRow(db, rows)
	if err != nil {
		return nil
	}
	return r
}

func GetTagFromRow(db *Database, row *sql.Rows) (*Tag, error) {
	r := &Tag{
		Type: Type{
			db: db,
		},
	}
	err := row.Scan(&r.id, &r.name)

	if err != nil {
		return nil, err
	}
	return r, nil
}

func (r *Tag) GetId() int64 {
	return r.id
}

func (r *Tag) GetName() string {
	return r.name
}

func (r *Tag) GetAlbums() (albums []*Album) {
	rows, err := r.db.Query("SELECT id, cover, identifiers, metadata FROM albums WHERE id IN(SELECT album FROM album_taggings WHERE tag = $1);", r.id)
	if err != nil {
		return nil
	}
	defer rows.Close()
	for rows.Next() {
		album, err := GetAlbumFromRow(r.db, rows)
		if err != nil {
			break
		}
		albums = append(albums, album)
	}
	return
}
